@@ -12,7 +12,6 @@
 #include <zend_smart_str.h>
 #include <ext/standard/php_smart_string.h>
 
-#define PHPREDIS_ZVAL_IS_STRICT_FALSE(z) (Z_TYPE_P(z) == IS_FALSE)
 #define PHPREDIS_GET_OBJECT(class_entry, o) (class_entry *)((char *)o - XtOffsetOf(class_entry, std))
 #define PHPREDIS_ZVAL_GET_OBJECT(class_entry, z) PHPREDIS_GET_OBJECT(class_entry, Z_OBJ_P(z))
 
@@ -21,6 +20,8 @@
 #define NULL   ((void *) 0)
 #endif
 
+#include "backoff.h"
+
 typedef enum {
     REDIS_SOCK_STATUS_FAILED = -1,
     REDIS_SOCK_STATUS_DISCONNECTED,
@@ -83,6 +84,10 @@ typedef enum _PUBSUB_TYPE {
 #define REDIS_OPT_REPLY_LITERAL      8
 #define REDIS_OPT_COMPRESSION_LEVEL  9
 #define REDIS_OPT_NULL_MBULK_AS_NULL 10
+#define REDIS_OPT_MAX_RETRIES        11
+#define REDIS_OPT_BACKOFF_ALGORITHM  12
+#define REDIS_OPT_BACKOFF_BASE       13
+#define REDIS_OPT_BACKOFF_CAP        14
 
 /* cluster options */
 #define REDIS_FAILOVER_NONE              0
@@ -109,6 +114,16 @@ typedef enum {
 #define REDIS_SCAN_PREFIX  2
 #define REDIS_SCAN_NOPREFIX 3
 
+/* BACKOFF_ALGORITHM options */
+#define REDIS_BACKOFF_ALGORITHMS                    7
+#define REDIS_BACKOFF_ALGORITHM_DEFAULT             0
+#define REDIS_BACKOFF_ALGORITHM_DECORRELATED_JITTER 1
+#define REDIS_BACKOFF_ALGORITHM_FULL_JITTER         2
+#define REDIS_BACKOFF_ALGORITHM_EQUAL_JITTER        3
+#define REDIS_BACKOFF_ALGORITHM_EXPONENTIAL         4
+#define REDIS_BACKOFF_ALGORITHM_UNIFORM             5
+#define REDIS_BACKOFF_ALGORITHM_CONSTANT            6
+
 /* GETBIT/SETBIT offset range limits */
 #define BITOP_MIN_OFFSET 0
 #define BITOP_MAX_OFFSET 4294967295U
@@ -118,6 +133,17 @@ typedef enum {
 #define MULTI    1
 #define PIPELINE 2
 
+#define PHPREDIS_DEBUG_LOGGING 0
+
+#if PHPREDIS_DEBUG_LOGGING == 1
+#define redisDbgFmt(fmt, ...) \
+    php_printf("%s:%d:%s(): " fmt "\n", __FILE__, __LINE__, __func__, __VA_ARGS__)
+#define redisDbgStr(str) phpredisDebugFmt("%s", str)
+#else
+#define redisDbgFmt(fmt, ...) ((void)0)
+#define redisDbgStr(str) ((void)0)
+#endif
+
 #define IS_ATOMIC(redis_sock) (redis_sock->mode == ATOMIC)
 #define IS_MULTI(redis_sock) (redis_sock->mode & MULTI)
 #define IS_PIPELINE(redis_sock) (redis_sock->mode & PIPELINE)
@@ -140,7 +166,7 @@ typedef enum {
 
 #define REDIS_SAVE_CALLBACK(callback, closure_context) do { \
     fold_item *fi = malloc(sizeof(fold_item)); \
-    fi->fun = (void *)callback; \
+    fi->fun = callback; \
     fi->ctx = closure_context; \
     fi->next = NULL; \
     if (redis_sock->current) { \
@@ -195,7 +221,7 @@ typedef enum {
         REDIS_PROCESS_RESPONSE_CLOSURE(resp_func, ctx) \
     }
 
-/* Process a command but with a specific command building function 
+/* Process a command but with a specific command building function
  * and keyword which is passed to us*/
 #define REDIS_PROCESS_KW_CMD(kw, cmdfunc, resp_func) \
     RedisSock *redis_sock; char *cmd; int cmd_len; void *ctx=NULL; \
@@ -256,52 +282,58 @@ typedef enum {
     #endif
 #endif
 
+/* {{{ struct RedisSock */
+typedef struct {
+    php_stream          *stream;
+    php_stream_context  *stream_ctx;
+    zend_string         *host;
+    int                 port;
+    zend_string         *user;
+    zend_string         *pass;
+    double              timeout;
+    double              read_timeout;
+    long                retry_interval;
+    int                 max_retries;
+    struct RedisBackoff backoff;
+    redis_sock_status   status;
+    int                 persistent;
+    int                 watching;
+    zend_string         *persistent_id;
+
+    redis_serializer    serializer;
+    int                 compression;
+    int                 compression_level;
+    long                dbNumber;
+
+    zend_string         *prefix;
+
+    short               mode;
+    struct fold_item    *head;
+    struct fold_item    *current;
+
+    zend_string         *pipeline_cmd;
+
+    zend_string         *err;
+
+    int                 scan;
+
+    int                 readonly;
+    int                 reply_literal;
+    int                 null_mbulk_as_null;
+    int                 tcp_keepalive;
+} RedisSock;
+/* }}} */
+
+/* Redis response handler function callback prototype */
+typedef void (*ResultCallback)(INTERNAL_FUNCTION_PARAMETERS, RedisSock *redis_sock, zval *z_tab, void *ctx);
+typedef int (*FailableResultCallback)(INTERNAL_FUNCTION_PARAMETERS, RedisSock*, zval*, void*);
+
 typedef struct fold_item {
-    zval * (*fun)(INTERNAL_FUNCTION_PARAMETERS, void *, ...);
+    FailableResultCallback fun;
     void *ctx;
     struct fold_item *next;
 } fold_item;
 
-/* {{{ struct RedisSock */
-typedef struct {
-    php_stream         *stream;
-    php_stream_context *stream_ctx;
-    zend_string        *host;
-    int                port;
-    zend_string        *user;
-    zend_string        *pass;
-    double             timeout;
-    double             read_timeout;
-    long               retry_interval;
-    redis_sock_status  status;
-    int                persistent;
-    int                watching;
-    zend_string        *persistent_id;
-
-    redis_serializer   serializer;
-    int                compression;
-    int                compression_level;
-    long               dbNumber;
-
-    zend_string        *prefix;
-
-    short              mode;
-    fold_item          *head;
-    fold_item          *current;
-
-    zend_string        *pipeline_cmd;
-
-    zend_string        *err;
-
-    int                scan;
-
-    int                readonly;
-    int                reply_literal;
-    int                null_mbulk_as_null;
-    int                tcp_keepalive;
-} RedisSock;
-/* }}} */
-
 typedef struct {
     zend_llist list;
     int nb_active;
