@@ -20,18 +20,16 @@
 #endif
 
 #include "common.h"
-#include "ext/standard/info.h"
-#include "php_ini.h"
-#include "php_redis.h"
-#include <zend_exceptions.h>
-
 #include "library.h"
 #include "redis_array.h"
 #include "redis_array_impl.h"
 
+#include <ext/standard/info.h>
+#include <zend_exceptions.h>
+
 /* Simple macro to detect failure in a RedisArray call */
 #define RA_CALL_FAILED(rv, cmd) ( \
-    PHPREDIS_ZVAL_IS_STRICT_FALSE(rv) || \
+    (Z_TYPE_P(rv) == IS_FALSE) || \
     (Z_TYPE_P(rv) == IS_ARRAY && zend_hash_num_elements(Z_ARRVAL_P(rv)) == 0) || \
     (Z_TYPE_P(rv) == IS_LONG && Z_LVAL_P(rv) == 0 && !strcasecmp(cmd, "TYPE")) \
 )
@@ -228,7 +226,7 @@ PHP_METHOD(RedisArray, __construct)
     RedisArray *ra = NULL;
     zend_bool b_index = 0, b_autorehash = 0, b_pconnect = 0, consistent = 0;
     HashTable *hPrev = NULL, *hOpts = NULL;
-    long l_retry_interval = 0;
+    zend_long l_retry_interval = 0;
       zend_bool b_lazy_connect = 0;
     double d_connect_timeout = 0, read_timeout = 0.0;
     zend_string *algorithm = NULL, *user = NULL, *pass = NULL;
@@ -242,8 +240,14 @@ PHP_METHOD(RedisArray, __construct)
      * Note:  WRONG_PARAM_COUNT seems wrong but this is what we have been doing
      *        for ages so we can't really change it until the next major version.
      */
-    if (Z_TYPE_P(z0) != IS_ARRAY && Z_TYPE_P(z0) != IS_STRING)
+    if (Z_TYPE_P(z0) != IS_ARRAY && Z_TYPE_P(z0) != IS_STRING) {
+#if PHP_VERSION_ID < 80000
         WRONG_PARAM_COUNT;
+#else
+        zend_argument_type_error(1, "must be of type string|array, %s given", zend_zval_type_name(z0));
+        RETURN_THROWS();
+#endif
+    }
 
     /* If it's a string we want to load the array from ini information */
     if (Z_TYPE_P(z0) == IS_STRING) {
@@ -338,7 +342,7 @@ ra_forward_call(INTERNAL_FUNCTION_PARAMETERS, RedisArray *ra, const char *cmd,
 
     /* pass call through */
     ZVAL_STRINGL(&z_fun, cmd, cmd_len); /* method name */
-    z_callargs = ecalloc(argc, sizeof(zval));
+    z_callargs = ecalloc(argc, sizeof(*z_callargs));
 
     /* copy args to array */
     i = 0;
@@ -475,12 +479,11 @@ PHP_METHOD(RedisArray, _instance)
 {
     zval *object;
     RedisArray *ra;
-    char *target;
-    size_t target_len;
+    zend_string *host;
     zval *z_redis;
 
-    if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os",
-                &object, redis_array_ce, &target, &target_len) == FAILURE) {
+    if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "OS",
+                &object, redis_array_ce, &host) == FAILURE) {
         RETURN_FALSE;
     }
 
@@ -488,17 +491,15 @@ PHP_METHOD(RedisArray, _instance)
         RETURN_FALSE;
     }
 
-    z_redis = ra_find_node_by_name(ra, target, target_len);
-    if(z_redis) {
-        RETURN_ZVAL(z_redis, 1, 0);
-    } else {
+    if ((z_redis = ra_find_node_by_name(ra, host)) == NULL) {
         RETURN_NULL();
     }
+    RETURN_ZVAL(z_redis, 1, 0);
 }
 
 PHP_METHOD(RedisArray, _function)
 {
-    zval *object, *z_fun;
+    zval *object;
     RedisArray *ra;
 
     if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "O",
@@ -510,13 +511,12 @@ PHP_METHOD(RedisArray, _function)
         RETURN_FALSE;
     }
 
-    z_fun = &ra->z_fun;
-    RETURN_ZVAL(z_fun, 1, 0);
+    RETURN_ZVAL(&ra->z_fun, 1, 0);
 }
 
 PHP_METHOD(RedisArray, _distributor)
 {
-    zval *object, *z_dist;
+    zval *object;
     RedisArray *ra;
 
     if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "O",
@@ -528,8 +528,7 @@ PHP_METHOD(RedisArray, _distributor)
         RETURN_FALSE;
     }
 
-    z_dist = &ra->z_dist;
-    RETURN_ZVAL(z_dist, 1, 0);
+    RETURN_ZVAL(&ra->z_dist, 1, 0);
 }
 
 PHP_METHOD(RedisArray, _rehash)
@@ -818,12 +817,10 @@ PHP_METHOD(RedisArray, select)
 /* MGET will distribute the call to several nodes and regroup the values. */
 PHP_METHOD(RedisArray, mget)
 {
-    zval *object, *z_keys, z_argarray, *data, z_ret, *z_cur, z_tmp_array;
-    int i, j, n;
-    RedisArray *ra;
-    int *pos, argc, *argc_each;
+    zval *object, *z_keys, *data, z_ret, *z_cur, z_tmp_array, z_fun, z_arg, **argv;
+    int i, j, n, *pos, argc, *argc_each;
     HashTable *h_keys;
-    zval **argv;
+    RedisArray *ra;
 
     if ((ra = redis_array_get(getThis())) == NULL) {
         RETURN_FALSE;
@@ -843,11 +840,10 @@ PHP_METHOD(RedisArray, mget)
     if ((argc = zend_hash_num_elements(h_keys)) == 0) {
         RETURN_FALSE;
     }
-    argv = emalloc(argc * sizeof(zval*));
-    pos = emalloc(argc * sizeof(int));
+    argv = ecalloc(argc, sizeof(*argv));
+    pos = ecalloc(argc, sizeof(*pos));
 
-    argc_each = emalloc(ra->count * sizeof(int));
-    memset(argc_each, 0, ra->count * sizeof(int));
+    argc_each = ecalloc(ra->count, sizeof(*argc_each));
 
     /* associate each key to a redis node */
     i = 0;
@@ -859,95 +855,87 @@ PHP_METHOD(RedisArray, mget)
         /* Handle the possibility that we're a reference */
         ZVAL_DEREF(data);
 
-        /* phpredis proper can only use string or long keys, so restrict to that here */
-        if (Z_TYPE_P(data) != IS_STRING && Z_TYPE_P(data) != IS_LONG) {
-            php_error_docref(NULL, E_ERROR, "MGET: all keys must be strings or longs");
-            efree(argv);
-            efree(pos);
-            efree(argc_each);
-            RETURN_FALSE;
-        }
-
         /* Convert to a string for hash lookup if it isn't one */
         if (Z_TYPE_P(data) == IS_STRING) {
             key_len = Z_STRLEN_P(data);
             key_lookup = Z_STRVAL_P(data);
-        } else {
+        } else if (Z_TYPE_P(data) == IS_LONG) {
             key_len = snprintf(kbuf, sizeof(kbuf), ZEND_LONG_FMT, Z_LVAL_P(data));
             key_lookup = (char*)kbuf;
+        } else {
+            /* phpredis proper can only use string or long keys, so restrict to that here */
+            php_error_docref(NULL, E_ERROR, "MGET: all keys must be strings or longs");
+            RETVAL_FALSE;
+            goto cleanup;
         }
 
         /* Find our node */
         if (ra_find_node(ra, key_lookup, key_len, &pos[i]) == NULL) {
-            /* TODO: handle */
+            RETVAL_FALSE;
+            goto cleanup;
         }
 
         argc_each[pos[i]]++;    /* count number of keys per node */
         argv[i++] = data;
     } ZEND_HASH_FOREACH_END();
 
+    /* prepare call */
     array_init(&z_tmp_array);
+    ZVAL_STRINGL(&z_fun, "MGET", sizeof("MGET") - 1);
+
     /* calls */
     for(n = 0; n < ra->count; ++n) { /* for each node */
         /* We don't even need to make a call to this node if no keys go there */
         if(!argc_each[n]) continue;
 
         /* copy args for MGET call on node. */
-        array_init(&z_argarray);
+        array_init(&z_arg);
 
         for(i = 0; i < argc; ++i) {
-            if(pos[i] != n) continue;
-
-            zval z_ret;
-            ZVAL_ZVAL(&z_ret, argv[i], 1, 0);
-            add_next_index_zval(&z_argarray, &z_ret);
+            if (pos[i] == n) {
+                ZVAL_ZVAL(&z_ret, argv[i], 1, 0);
+                add_next_index_zval(&z_arg, &z_ret);
+            }
         }
 
-        zval z_fun;
-        /* prepare call */
-        ZVAL_STRINGL(&z_fun, "MGET", 4);
         /* call MGET on the node */
-        call_user_function(&redis_ce->function_table, &ra->redis[n], &z_fun, &z_ret, 1, &z_argarray);
-        zval_dtor(&z_fun);
+        call_user_function(&redis_ce->function_table, &ra->redis[n], &z_fun, &z_ret, 1, &z_arg);
 
         /* cleanup args array */
-        zval_dtor(&z_argarray);
+        zval_dtor(&z_arg);
 
         /* Error out if we didn't get a proper response */
         if (Z_TYPE(z_ret) != IS_ARRAY) {
             /* cleanup */
             zval_dtor(&z_ret);
             zval_dtor(&z_tmp_array);
-            efree(argv);
-            efree(pos);
-            efree(argc_each);
-
-            /* failure */
-            RETURN_FALSE;
+            RETVAL_FALSE;
+            goto cleanup;
         }
 
         for(i = 0, j = 0; i < argc; ++i) {
             if (pos[i] != n || (z_cur = zend_hash_index_find(Z_ARRVAL(z_ret), j++)) == NULL) continue;
 
-            zval z_ret;
-            ZVAL_ZVAL(&z_ret, z_cur, 1, 0);
-            add_index_zval(&z_tmp_array, i, &z_ret);
+            ZVAL_ZVAL(&z_arg, z_cur, 1, 0);
+            add_index_zval(&z_tmp_array, i, &z_arg);
         }
         zval_dtor(&z_ret);
     }
 
+    zval_dtor(&z_fun);
+
     array_init(return_value);
     /* copy temp array in the right order to return_value */
     for(i = 0; i < argc; ++i) {
         if ((z_cur = zend_hash_index_find(Z_ARRVAL(z_tmp_array), i)) == NULL) continue;
 
-        zval z_ret;
-        ZVAL_ZVAL(&z_ret, z_cur, 1, 0);
-        add_next_index_zval(return_value, &z_ret);
+        ZVAL_ZVAL(&z_arg, z_cur, 1, 0);
+        add_next_index_zval(return_value, &z_arg);
     }
 
     /* cleanup */
     zval_dtor(&z_tmp_array);
+cleanup:
     efree(argv);
     efree(pos);
     efree(argc_each);
@@ -957,13 +945,11 @@ PHP_METHOD(RedisArray, mget)
 /* MSET will distribute the call to several nodes and regroup the values. */
 PHP_METHOD(RedisArray, mset)
 {
-    zval *object, *z_keys, z_argarray, *data, z_ret, **argv;
-    int i = 0, n;
+    zval *object, *z_keys, z_argarray, *data, z_fun, z_ret, **argv;
+    int i = 0, n, *pos, argc, *argc_each, key_len;
     RedisArray *ra;
-    int *pos, argc, *argc_each;
     HashTable *h_keys;
     char *key, kbuf[40];
-    int key_len;
     zend_string **keys, *zkey;
     zend_ulong idx;
 
@@ -985,12 +971,11 @@ PHP_METHOD(RedisArray, mset)
     if ((argc = zend_hash_num_elements(h_keys)) == 0) {
         RETURN_FALSE;
     }
-    argv = emalloc(argc * sizeof(zval*));
-    pos = emalloc(argc * sizeof(int));
-    keys = ecalloc(argc, sizeof(zend_string *));
+    argv = ecalloc(argc, sizeof(*argv));
+    pos = ecalloc(argc, sizeof(*pos));
+    keys = ecalloc(argc, sizeof(*keys));
 
-    argc_each = emalloc(ra->count * sizeof(int));
-    memset(argc_each, 0, ra->count * sizeof(int));
+    argc_each = ecalloc(ra->count, sizeof(*argc_each));
 
     /* associate each key to a redis node */
     ZEND_HASH_FOREACH_KEY_VAL(h_keys, idx, zkey, data) {
@@ -1004,16 +989,26 @@ PHP_METHOD(RedisArray, mset)
         }
 
         if (ra_find_node(ra, key, (int)key_len, &pos[i]) == NULL) {
-            // TODO: handle
+            for (n = 0; n < i; ++n) {
+                zend_string_release(keys[n]);
+            }
+            efree(keys);
+            efree(argv);
+            efree(pos);
+            efree(argc_each);
+            RETURN_FALSE;
         }
 
         argc_each[pos[i]]++;    /* count number of keys per node */
-        keys[i] = zend_string_init(key, key_len, 0);
+        keys[i] = zkey ? zend_string_copy(zkey) : zend_string_init(key, key_len, 0);
         argv[i] = data;
         i++;
     } ZEND_HASH_FOREACH_END();
 
 
+    /* prepare call */
+    ZVAL_STRINGL(&z_fun, "MSET", sizeof("MSET") - 1);
+
     /* calls */
     for (n = 0; n < ra->count; ++n) { /* for each node */
         /* We don't even need to make a call to this node if no keys go there */
@@ -1026,7 +1021,6 @@ PHP_METHOD(RedisArray, mset)
         for(i = 0; i < argc; ++i) {
             if(pos[i] != n) continue;
 
-            zval z_ret;
             if (argv[i] == NULL) {
                 ZVAL_NULL(&z_ret);
             } else {
@@ -1043,26 +1037,19 @@ PHP_METHOD(RedisArray, mset)
 
         if(ra->index) { /* add MULTI */
             ra_index_multi(&ra->redis[n], MULTI);
-        }
-
-        zval z_fun;
-
-        /* prepare call */
-        ZVAL_STRINGL(&z_fun, "MSET", 4);
-
-        /* call */
-        call_user_function(&redis_ce->function_table, &ra->redis[n], &z_fun, &z_ret, 1, &z_argarray);
-        zval_dtor(&z_fun);
-        zval_dtor(&z_ret);
-
-        if(ra->index) {
+            call_user_function(&redis_ce->function_table, &ra->redis[n], &z_fun, &z_ret, 1, &z_argarray);
             ra_index_keys(&z_argarray, &ra->redis[n]); /* use SADD to add keys to node index */
             ra_index_exec(&ra->redis[n], NULL, 0); /* run EXEC */
+        } else {
+            call_user_function(&redis_ce->function_table, &ra->redis[n], &z_fun, &z_ret, 1, &z_argarray);
         }
 
         zval_dtor(&z_argarray);
+        zval_dtor(&z_ret);
     }
 
+    zval_dtor(&z_fun);
+
     /* Free any keys that we needed to allocate memory for, because they weren't strings */
     for(i = 0; i < argc; i++) {
         zend_string_release(keys[i]);
@@ -1078,15 +1065,14 @@ PHP_METHOD(RedisArray, mset)
 }
 
 /* Generic handler for DEL or UNLINK which behave identically to phpredis */
-static void ra_generic_del(INTERNAL_FUNCTION_PARAMETERS, char *kw, int kw_len) {
-    zval *object, z_keys, z_fun, *data, z_ret, *z_args;
-    int i, n;
-    RedisArray *ra;
-    int *pos, argc = ZEND_NUM_ARGS(), *argc_each;
+static void
+ra_generic_del(INTERNAL_FUNCTION_PARAMETERS, char *kw, int kw_len)
+{
+    zval *object, z_keys, z_fun, *data, z_ret, *z_args, **argv;
+    int i, n, *pos, argc = ZEND_NUM_ARGS(), *argc_each, free_zkeys = 0;
     HashTable *h_keys;
-    zval **argv;
+    RedisArray *ra;
     long total = 0;
-    int free_zkeys = 0;
 
     if ((ra = redis_array_get(getThis())) == NULL) {
         RETURN_FALSE;
@@ -1096,7 +1082,7 @@ static void ra_generic_del(INTERNAL_FUNCTION_PARAMETERS, char *kw, int kw_len) {
     HANDLE_MULTI_EXEC(ra, kw, kw_len);
 
     /* get all args in z_args */
-    z_args = emalloc(argc * sizeof(zval));
+    z_args = ecalloc(argc, sizeof(*z_args));
     if (zend_get_parameters_array(ht, argc, z_args) == FAILURE) {
         efree(z_args);
         RETURN_FALSE;
@@ -1109,11 +1095,7 @@ static void ra_generic_del(INTERNAL_FUNCTION_PARAMETERS, char *kw, int kw_len) {
         /* copy all elements to z_keys */
         array_init(&z_keys);
         for (i = 0; i < argc; ++i) {
-            zval *z_arg = &z_args[i];
-            zval z_ret;
-            ZVAL_ZVAL(&z_ret, z_arg, 1, 0);
-            /* add copy to z_keys */
-            add_next_index_zval(&z_keys, &z_ret);
+            add_next_index_zval(&z_keys, &z_args[i]);
         }
         free_zkeys = 1;
     }
@@ -1125,27 +1107,23 @@ static void ra_generic_del(INTERNAL_FUNCTION_PARAMETERS, char *kw, int kw_len) {
         efree(z_args);
         RETURN_FALSE;
     }
-    argv = emalloc(argc * sizeof(zval*));
-    pos = emalloc(argc * sizeof(int));
+    argv = ecalloc(argc, sizeof(*argv));
+    pos = ecalloc(argc, sizeof(*pos));
 
-    argc_each = emalloc(ra->count * sizeof(int));
-    memset(argc_each, 0, ra->count * sizeof(int));
+    argc_each = ecalloc(ra->count, sizeof(*argc_each));
 
     /* associate each key to a redis node */
     i = 0;
     ZEND_HASH_FOREACH_VAL(h_keys, data) {
         if (Z_TYPE_P(data) != IS_STRING) {
             php_error_docref(NULL, E_ERROR, "DEL: all keys must be string.");
-            if (free_zkeys) zval_dtor(&z_keys);
-            efree(z_args);
-            efree(argv);
-            efree(pos);
-            efree(argc_each);
-            RETURN_FALSE;
+            RETVAL_FALSE;
+            goto cleanup;
         }
 
         if (ra_find_node(ra, Z_STRVAL_P(data), Z_STRLEN_P(data), &pos[i]) == NULL) {
-            // TODO: handle
+            RETVAL_FALSE;
+            goto cleanup;
         }
         argc_each[pos[i]]++;    /* count number of keys per node */
         argv[i++] = data;
@@ -1165,12 +1143,10 @@ static void ra_generic_del(INTERNAL_FUNCTION_PARAMETERS, char *kw, int kw_len) {
         /* copy args */
         array_init(&z_argarray);
         for(i = 0; i < argc; ++i) {
-            if(pos[i] != n) continue;
-
-            zval z_ret;
-            ZVAL_ZVAL(&z_ret, argv[i], 1, 0);
-            add_next_index_zval(&z_argarray, &z_ret);
-            found++;
+            if (pos[i] == n) {
+                add_next_index_zval(&z_argarray, argv[i]);
+                found++;
+            }
         }
 
         if(!found) {    /* don't run empty DEL or UNLINK commands */
@@ -1180,15 +1156,11 @@ static void ra_generic_del(INTERNAL_FUNCTION_PARAMETERS, char *kw, int kw_len) {
 
         if(ra->index) { /* add MULTI */
             ra_index_multi(&ra->redis[n], MULTI);
-        }
-
-        /* call */
-        call_user_function(&redis_ce->function_table, &ra->redis[n], &z_fun, &z_ret, 1, &z_argarray);
-
-        if(ra->index) {
-            zval_dtor(&z_ret);
+            call_user_function(&redis_ce->function_table, &ra->redis[n], &z_fun, &z_ret, 1, &z_argarray);
             ra_index_del(&z_argarray, &ra->redis[n]); /* use SREM to remove keys from node index */
             ra_index_exec(&ra->redis[n], &z_ret, 0); /* run EXEC */
+        } else {
+            call_user_function(&redis_ce->function_table, &ra->redis[n], &z_fun, &z_ret, 1, &z_argarray);
         }
         total += Z_LVAL(z_ret);    /* increment total */
 
@@ -1196,8 +1168,11 @@ static void ra_generic_del(INTERNAL_FUNCTION_PARAMETERS, char *kw, int kw_len) {
         zval_dtor(&z_ret);
     }
 
-    /* cleanup */
     zval_dtor(&z_fun);
+
+    RETVAL_LONG(total);
+
+cleanup:
     efree(argv);
     efree(pos);
     efree(argc_each);
@@ -1205,9 +1180,7 @@ static void ra_generic_del(INTERNAL_FUNCTION_PARAMETERS, char *kw, int kw_len) {
     if(free_zkeys) {
         zval_dtor(&z_keys);
     }
-
     efree(z_args);
-    RETURN_LONG(total);
 }
 
 /* DEL will distribute the call to several nodes and regroup the values. */
@@ -1225,12 +1198,11 @@ PHP_METHOD(RedisArray, multi)
     zval *object;
     RedisArray *ra;
     zval *z_redis;
-    char *host;
-    size_t host_len;
+    zend_string *host;
     zend_long multi_value = MULTI;
 
-    if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os|l",
-                &object, redis_array_ce, &host, &host_len, &multi_value) == FAILURE) {
+    if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "OS|l",
+                &object, redis_array_ce, &host, &multi_value) == FAILURE) {
         RETURN_FALSE;
     }
 
@@ -1239,8 +1211,7 @@ PHP_METHOD(RedisArray, multi)
     }
 
     /* find node */
-    z_redis = ra_find_node_by_name(ra, host, host_len);
-    if(!z_redis) {
+    if ((z_redis = ra_find_node_by_name(ra, host)) == NULL) {
         RETURN_FALSE;
     }
 
